/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.EnumMap;
import java.util.List;
import nl.cloudfarming.client.model.SensorDataFile;
import org.openide.util.NbBundle;

/**
 *
 * @author Gerben Feenstra
 */
public class GreenseekerFileProcessorBorne extends GreenseekerFileProcessor {

    public GreenseekerFileProcessorBorne(File fileToProcess) {
        super(fileToProcess);
    }
    // Constant int values
    private static final String LOGKEY_TYPE_DISPLAYNAME = NbBundle.getMessage(GreenseekerFileProcessorBorne.class, "sensor.greenseeker.fileprocessor.borne.log.displayname");
    private static final SimpleDateFormat TIME_FORMAT = new SimpleDateFormat("HHmmss.SS");

    /**
     * Parse Greenseeker file, return list of raw records.
     * @return A list of raw records
     * @throws FileNotFoundException
     * @throws IOException 
     */
    @Override
    protected List<String> parseFile(File greenseekerBorneFile) throws FileNotFoundException, IOException {
        List<String> recordList = new ArrayList<String>();

        // Create new BufferedInputStream
        BufferedInputStream bis = new BufferedInputStream(new FileInputStream(greenseekerBorneFile));

        // Create record byte[] container
        final byte[] firstRecord = new byte[GreenseekerFileDescriptionBorne.RECORD_LENGTH + 1]; // Borne firstline contains a return \r
        final byte[] record = new byte[GreenseekerFileDescriptionBorne.RECORD_LENGTH];

        // Skip header
        bis.skip(GreenseekerFileDescriptionBorne.HEADER_LENGTH);

        // First record contains return \r, so bytearray should be borne recordlength + 1
        bis.read(firstRecord);
        String firstrecStr = new String(firstRecord);
        recordList.add(firstrecStr);

        // Other records
        while (bis.read(record) > 0) {
            String rec = new String(record);
            recordList.add(rec);
        }
        return recordList;
    }

    /**
     * Parses one record of a v/d Borne Greenseeker sensorfile.
     *
     * @param payload The record to be processed.
     * @param dataFile The SensorDataFile belonging to this sensor file.
     * @return A record represented as a Greenseeker sensordata object
     * @throws ParseException Exception occurs when date/time cannot be parsed.
     */
    @Override
    protected GreenseekerSensorData parseLine(String payload, SensorDataFile dataFile) throws GreenseekerParseException {
        // Split payload into fields-array
        String[] fields = getFields(payload);

        EnumMap map = new EnumMap<GreenseekerFileDescriptionBorne, Object>(GreenseekerFileDescriptionBorne.class);
        try {
            for (GreenseekerFileDescriptionBorne greenseekerBorneField : GreenseekerFileDescriptionBorne.values()) {
                if (greenseekerBorneField.isMandatory()) {
                    if (greenseekerBorneField.getTheClass().equals(java.lang.Double.class)) {
                        double theDouble = Double.parseDouble(fields[greenseekerBorneField.getPosition()]);
                        map.put(greenseekerBorneField, theDouble);
                    } else if (greenseekerBorneField.getTheClass().equals(java.lang.Integer.class)) {
                        int theInt = Integer.parseInt(fields[greenseekerBorneField.getPosition()]);
                        map.put(greenseekerBorneField, theInt);
                    } else if (greenseekerBorneField.getTheClass().equals(java.util.Date.class)) {
                        Date theDate = TIME_FORMAT.parse(fields[greenseekerBorneField.getPosition()]);
                        map.put(greenseekerBorneField, theDate);
                    } else {
                        map.put(greenseekerBorneField, fields[greenseekerBorneField.getPosition()]);
                    }
                }
            }
        } catch (ParseException e) {
            throw new GreenseekerParseException("Error parsing date ", e);
        } catch (Exception e) {
            throw new GreenseekerParseException(e);
        }

        GreenseekerSensorData sensorData = new GreenseekerSensorData();
        sensorData.setSensorDataFile(dataFile);
        sensorData.setObjectId((Double) map.get(GreenseekerFileDescriptionBorne.OBJECTID));
        sensorData.setLongitude((Double) map.get(GreenseekerFileDescriptionBorne.LONGITUDE));
        sensorData.setLatitude((Double) map.get(GreenseekerFileDescriptionBorne.LATITUDE));
        sensorData.setSpeed((Double) map.get(GreenseekerFileDescriptionBorne.SPEED));
        sensorData.setHeading((Double) map.get(GreenseekerFileDescriptionBorne.HEADING));
        sensorData.setElevation((Double) map.get(GreenseekerFileDescriptionBorne.ELEVATION));

        sensorData.setNdvi((Double) map.get(GreenseekerFileDescriptionBorne.NDVI));
        sensorData.setVi2nd((Double) map.get(GreenseekerFileDescriptionBorne.VI2ND));

        sensorData.setRed54((Double) map.get(GreenseekerFileDescriptionBorne.RED54));
        sensorData.setNir54((Double) map.get(GreenseekerFileDescriptionBorne.NIR54));

        sensorData.setRed55((Double) map.get(GreenseekerFileDescriptionBorne.RED55));
        sensorData.setNir55((Double) map.get(GreenseekerFileDescriptionBorne.NIR55));

        sensorData.setRed57((Double) map.get(GreenseekerFileDescriptionBorne.RED57));
        sensorData.setNir57((Double) map.get(GreenseekerFileDescriptionBorne.NIR57));

        sensorData.setRed53((Double) map.get(GreenseekerFileDescriptionBorne.RED53));
        sensorData.setNir53((Double) map.get(GreenseekerFileDescriptionBorne.NIR53));

        sensorData.setRed56((Double) map.get(GreenseekerFileDescriptionBorne.RED56));
        sensorData.setNir56((Double) map.get(GreenseekerFileDescriptionBorne.NIR56));
        sensorData.setRed58((Double) map.get(GreenseekerFileDescriptionBorne.RED58));
        sensorData.setNir58((Double) map.get(GreenseekerFileDescriptionBorne.NIR58));

        return sensorData;
    }

    /**
     * Concrete subclasses must override this method to return the displayname of that sensorprocessor to be displayed in the log.
     * For example: "ASC-340 Cropcircle"
     * 
     * Primary use is for logging, for example: [Info] Start processing of ASC-340 Cropcircle sensor file 010710AD.CSV
     * 
     * @return 
     */
    @Override
    protected String getLogDisplayName() {
        return LOGKEY_TYPE_DISPLAYNAME;
    }
}
